import { EnvelopeIcon, MapPinIcon, PhoneIcon } from "@heroicons/react/24/solid";
import { GlobalContext } from "pages/_app";
import { Fragment, useCallback, useState } from "react";
import { useContext } from "react";
import { useGoogleReCaptcha } from "react-google-recaptcha-v3";
import { useForm } from "react-hook-form";
import { Notification } from "types";

import { Button } from "components/Button";
import { Input } from "components/Input";
import { Modal } from "components/Modal";
import { RenderContent } from "components/RenderContent";
import { Select } from "components/Select";
import { SocialLink } from "components/SocialLink";
import { Textarea } from "components/Textarea";

const ContactForm = ({ data }: any) => {
  const { socials, contactDetails } = useContext(GlobalContext);
  const { utilities } = data || {};
  const { id } = utilities || {};

  const { executeRecaptcha } = useGoogleReCaptcha();
  const [isOpen, setIsOpen] = useState<boolean>(false);
  const [notification, setNotification] = useState<Notification>();
  const { register, handleSubmit } = useForm();

  const onSubmit = useCallback(
    (data) => {
      if (!executeRecaptcha) {
        console.log("Execute recaptcha not yet available");
        return;
      }
      executeRecaptcha("enquiryFormSubmit").then((gReCaptchaToken) => {
        console.log(gReCaptchaToken, "response Google reCaptcha server");
        submitForm({ data, gReCaptchaToken });
      });
    },
    [executeRecaptcha]
  );

  const submitForm = ({ data, gReCaptchaToken }) => {
    fetch("/api/send-email", {
      method: "POST",
      headers: {
        Accept: "application/json, text/plain, */*",
        "Content-Type": "application/json",
      },
      body: JSON.stringify({
        ...data,
        gRecaptchaToken: gReCaptchaToken,
      }),
    })
      .then((res) => res.json())
      .then((res) => {
        console.log(res, "response from backend");
        if (res?.status === "success") {
          setNotification(res);
        } else {
          setNotification(res);
        }
        setIsOpen(true);
      });
  };

  const { emailAddress, telephoneNumber, address } = contactDetails;

  return (
    <Fragment>
      <div className="mx-auto top-full bg-white rounded-xl overflow-hidden w-full shadow-lg flex flex-col lg:flex-row p-2 lg:p-4">
        <div
          className="bg-[url('/images/blob-scene-haikei-2.svg')] p-6 py-12 lg:p-12 bg-cover bg-center shrink-0 lg:w-[40%] rounded-xl flex flex-col justify-between"
          id={id}
        >
          <div>
            <div className="mb-4 lg:mb-16">
              <p className="text-white font-display text-3xl mb-2 tracking-wider">
                Kontaktní informace
              </p>
              <p className="text-white text-sm lg:text-base">
                Vyplňte formulář a my se Vám brzy na vaši e-mailovou adresu ozveme.
              </p>
            </div>
            <ul className="text-white space-y-6 mb-8 lg:mb-32">
              <li className="flex items-center text-sm lg:text-base">
                <PhoneIcon className="h-5 w-5 mr-6 text-primary-200" />
                <span>{telephoneNumber}</span>
              </li>
              <li className="flex items-center text-sm lg:text-base">
                <EnvelopeIcon className="h-5 w-5 mr-6 text-primary-200" />
                <span>{emailAddress}</span>
              </li>
              <li className="flex items-center text-sm lg:text-base prose-p:text-white">
                <MapPinIcon className="h-5 w-5 mr-6 text-primary-200" />
                <RenderContent richText={address} />
              </li>
            </ul>
          </div>
          <div className="flex space-x-4">
            {socials.showFacebook && (
              <SocialLink
                href={socials.facebookUrl}
                icon={{ src: "/icons/facebook.svg", alt: "Facebook" }}
                label="Facebook"
              />
            )}
            {socials.showInstagram && (
              <SocialLink
                href={socials.instagramUrl}
                icon={{ src: "/icons/instagram.svg", alt: "Instagram" }}
                label="Instagram"
              />
            )}
            {socials.showTwitter && (
              <SocialLink
                href={socials.twitterUrl}
                icon={{ src: "/icons/twitter.svg", alt: "Twitter" }}
                label="Twutter"
              />
            )}
            {socials.showLinkedIn && (
              <SocialLink
                href={socials.linkedInUrl}
                icon={{ src: "/icons/linkedin-in.svg", alt: "LinkedIn" }}
                label="LinkedIn"
              />
            )}
          </div>
        </div>
        <form className="p-2 py-8 lg:p-12 w-full flex flex-col" onSubmit={handleSubmit(onSubmit)}>
          <div className="space-y-12 mb-8">
            <div className="grid lg:grid-cols-2 gap-8">
              <Input label="Jméno a příjmení" register={register} name="name" />
              <Input label="E-mailová adresa" register={register} name="email" />
            </div>
            <Select label="Mám zájem o následující produkt" register={register} name="type">
              <Select.Option value="fotovoltaika">Fotovoltaika</Select.Option>
              <Select.Option value="vytapeni">Vytápění</Select.Option>
              <Select.Option value="poradenstvi">Poradenství</Select.Option>
              <Select.Option value="dotace">Dotace a financování</Select.Option>
            </Select>
            <Input label="Předmět" register={register} name="subject" />
            <Textarea label="Zpráva" register={register} name="message" />
          </div>
          <Button size="base" style="primary" variant="primary" text="Odeslat" type="submit" />
        </form>
      </div>
      <Modal isOpen={isOpen} setIsOpen={setIsOpen} className="max-w-4xl" align="middle">
        <Modal.Title>{notification?.status === "success" ? "Úspěch" : "Chyba"}</Modal.Title>
        <Modal.Body>{notification?.message}</Modal.Body>
      </Modal>
    </Fragment>
  );
};

export default ContactForm;
