import { Disclosure, Tab } from "@headlessui/react";
import { CookieValueTypes, setCookie } from "cookies-next";
import { getCookie } from "cookies-next";
import Image from "next/image";
import { GlobalContext } from "pages/_app";
import { Fragment, useContext, useEffect, useState } from "react";

import { joinClassNames } from "utils/helper";

import { Button } from "components/Button";
import { Modal } from "components/Modal";
import { RenderContent } from "components/RenderContent";

const tabs = ["Souhlas", "Detaily", "Informace o cookies"];

export default function CookieConsent() {
  const { cookies } = useContext(GlobalContext);
  const [modalOpen, setModalOpen] = useState<boolean>(false);
  const [cookieStatus, setCookieStatus] = useState<CookieValueTypes>(undefined);
  const [cookieLoaded, setCookieLoaded] = useState<boolean>(false);

  const acceptCookies = () => {
    setCookie("cookieconsent_status", "allow", {
      maxAge: 60 * 60 * 24 * 30,
    });
    setCookieStatus("allow");
    setModalOpen(false);
    window.gtag("consent", "update", {
      ad_storage: "granted",
      analytics_storage: "granted",
    });
  };

  const denyCookies = () => {
    setCookie("cookieconsent_status", "deny", {
      maxAge: 60 * 60 * 24 * 30,
    });
    setCookieStatus("deny");
    setModalOpen(false);
    window.gtag("consent", "update", {
      ad_storage: "denied",
      analytics_storage: "denied",
    });
  };

  useEffect(() => {
    setCookieStatus(getCookie("cookieconsent_status"));
    setCookieLoaded(true);
  }, []);

  return (
    <>
      <div className="fixed bottom-4 right-4 z-50">
        <button
          className="h-10 w-10 bg-primary-600 hover:bg-primary-700 transition-colors flex justify-center items-center shadow rounded"
          onClick={() => setModalOpen(true)}
          title="Zobrazit lištu s cookies"
          aria-label="Zobrazit lištu s cookies"
        >
          <svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 512 512" className="h-4" fill="#fff">
            <path d="M231.2 17c-22.1-3.1-44.6 .9-64.4 11.4l-74 39.5C73.1 78.4 57.2 94.9 47.4 115L10.7 190.6c-9.8 20.1-13 42.9-9.1 64.9l14.5 82.8c3.9 22.1 14.6 42.3 30.7 57.9l60.3 58.4c16.1 15.6 36.6 25.6 58.7 28.7l83 11.7c22.1 3.1 44.6-.9 64.4-11.4l74-39.5c19.7-10.5 35.6-27 45.4-47.2l36.7-75.5c9.8-20.1 13-42.9 9.1-64.9l-14.6-82.8c-3.9-22.1-14.6-42.3-30.7-57.9L372.9 57.5c-16.1-15.6-36.6-25.6-58.7-28.7L231.2 17zM192 144a32 32 0 1 1 0 64 32 32 0 1 1 0-64zM128 336a32 32 0 1 1 64 0 32 32 0 1 1 -64 0zm224-64a32 32 0 1 1 0 64 32 32 0 1 1 0-64z" />
          </svg>
        </button>
      </div>
      <div className="flex gap-x-2">
        <Modal
          isOpen={
            (cookieStatus === undefined && cookieLoaded === true) ||
            (modalOpen && cookieStatus != undefined)
          }
          className="max-w-3xl"
          align="middle"
        >
          <Modal.Header>
            <Modal.Title>Nastavení cookies</Modal.Title>
          </Modal.Header>
          <Modal.Body>
            <Tab.Group>
              <Tab.List className="flex space-x-8 border-b mb-4">
                {tabs.map((item, index) => (
                  <Tab as={Fragment} key={index}>
                    {({ selected }) => (
                      <button
                        className={joinClassNames(
                          "font-display text-xl rounded border-0 outline-0 py-1 hover:text-primary-700",
                          selected ? "text-primary-600" : "text-gray-600"
                        )}
                      >
                        {item}
                      </button>
                    )}
                  </Tab>
                ))}
              </Tab.List>
              <Tab.Panels>
                <Tab.Panel>
                  <RenderContent richText={cookies?.agreement} />
                </Tab.Panel>
                <Tab.Panel>
                  <div className="flex flex-col divide-y">
                    {cookies?.categories?.map((item, index) => (
                      <Disclosure as="div" className="py-4 pt-0" key={index}>
                        <Disclosure.Button as={Fragment}>
                          <button className="flex space-x-4 py-2 items-center">
                            <Image
                              src="/icons/chevron-down-solid.svg"
                              width={0}
                              height={0}
                              alt="Ikona šipky dolů"
                              className="w-3"
                            />
                            <span>{item.title}</span>
                          </button>
                        </Disclosure.Button>
                        <Disclosure.Panel className="text-gray-500 ml-6">
                          <RenderContent richText={item.description} />
                        </Disclosure.Panel>
                      </Disclosure>
                    ))}
                  </div>
                </Tab.Panel>
                <Tab.Panel>
                  <RenderContent richText={cookies?.additionalInformation} />
                </Tab.Panel>
              </Tab.Panels>
            </Tab.Group>
            <p className="mb-4">
              Momentální stav cookies:{" "}
              <span
                className={joinClassNames(
                  "font-display text-xl uppercase",
                  cookieStatus === "allow" ? "text-primary-600" : "text-red-600"
                )}
              >
                {cookieStatus === "allow" ? "Povoleno" : "Zakázáno"}
              </span>
            </p>
          </Modal.Body>
          <Modal.Footer>
            <div className="flex space-x-4">
              <Button
                size="sm"
                text="Souhlasím"
                style="primary"
                variant="primary"
                onClick={acceptCookies}
              />
              <Button
                size="sm"
                text="Nesouhlasím"
                style="red"
                variant="primary"
                onClick={denyCookies}
              />
            </div>
          </Modal.Footer>
        </Modal>
      </div>
    </>
  );
}
