import { Dialog, Transition } from "@headlessui/react";
import { motion } from "framer-motion";
import Image from "next/image";
import { Media, Page } from "payload-types";
import { Fragment } from "react";
import { useEffect, useState } from "react";

import { animateOpacity, createImageUrl, joinClassNames } from "utils/helper";

type GalleryBlock = Extract<
  Extract<Page["layout"][number], { blockType: "container" }>["layout"][number],
  { blockType: "gallery" }
>;

const Gallery = ({ images }: GalleryBlock) => {
  const [modalOpen, setModalOpen] = useState(false);
  const [activeImage, setActiveImage] = useState(null);
  const [imageLoaded, setImageLoaded] = useState(false);

  useEffect(() => {
    if (modalOpen) {
      document.body.style.overflow = "hidden";
    } else {
      document.body.style.overflow = "auto";
    }
  }, [modalOpen]);

  useEffect(() => {
    if (modalOpen) {
      setImageLoaded(false);
    }
  }, [activeImage, modalOpen]);

  return (
    <Fragment>
      <div className="grid grid-cols-1 sm:grid-cols-2 xl:grid-cols-3 gap-8 items-stretch">
        {images.map((item, index) => (
          <motion.button
            className="relative h-[20rem]"
            key={index}
            onClick={() => {
              setModalOpen(!modalOpen);
              setActiveImage(item);
            }}
            {...animateOpacity}
          >
            <Image
              src={createImageUrl((item.image as Media).url)}
              alt={(item.image as Media).alt}
              className="object-cover hover:grayscale transition-all rounded-lg"
              sizes="(max-width: 768px) 100vw, (max-width: 1200px) 33vw, 20vw"
              fill
            />
          </motion.button>
        ))}
      </div>
      <Transition appear show={modalOpen} as={Fragment}>
        <Dialog as="div" className="relative z-40" onClose={() => setModalOpen(false)}>
          <Transition.Child
            as={Fragment}
            enter="ease-out duration-300"
            enterFrom="opacity-0"
            enterTo="opacity-100"
            leave="ease-in duration-200"
            leaveFrom="opacity-100"
            leaveTo="opacity-0"
          >
            <div className="fixed inset-0 bg-black bg-opacity-50" />
          </Transition.Child>

          <div className="fixed inset-0 overflow-y-auto">
            <div className="flex h-full items-center justify-center p-4 text-center">
              <Transition.Child
                as={Fragment}
                enter="ease-out duration-300"
                enterFrom="opacity-0 scale-95"
                enterTo="opacity-100 scale-100"
                leave="ease-in duration-200"
                leaveFrom="opacity-100 scale-100"
                leaveTo="opacity-0 scale-95"
              >
                <Dialog.Panel className="max-w-7xl transform overflow-hidden rounded text-center items-center transition-all flex justify-center">
                  <Image
                    src={createImageUrl(activeImage?.image.url)}
                    height={activeImage?.image.height}
                    width={activeImage?.image.width}
                    alt={activeImage?.alt}
                    onLoadingComplete={() => setImageLoaded(true)}
                    className={joinClassNames("shadow-xl", imageLoaded ? "visible" : "invisible")}
                  />
                  <Image
                    src="/icons/spinner-solid.svg"
                    alt="Ikona načítání"
                    width={0}
                    height={0}
                    className={joinClassNames(
                      "h-16 w-16 animate-spin absolute",
                      !imageLoaded ? "block" : "hidden"
                    )}
                  />
                </Dialog.Panel>
              </Transition.Child>
            </div>
          </div>
        </Dialog>
      </Transition>
    </Fragment>
  );
};

export default Gallery;
