import { Popover, Transition } from "@headlessui/react";
import Image from "next/image";
import Link from "next/link";
import { useRouter } from "next/router";
import { GlobalContext } from "pages/_app";
import { Media, Page, Post } from "payload-types";
import { Dispatch, Fragment, SetStateAction } from "react";
import { useContext } from "react";
import { useEffect } from "react";

import { createImageUrl, getAbsolutePath } from "utils/helper";
import { joinClassNames } from "utils/helper";

import { Modal } from "components/Modal";

type Props = {
  open: boolean;
  setOpen: Dispatch<SetStateAction<boolean>>;
  items: any;
};

const Menu = ({ open, setOpen, items }: Props) => {
  const { websiteSettings } = useContext(GlobalContext);
  const { events } = useRouter();

  const onRouteChange = () => {
    setOpen(false);
    document.body.style.overflow = "auto";
  };

  useEffect(() => {
    events.on("routeChangeStart", onRouteChange);
    return () => {
      events.off("routeChangeStart", onRouteChange);
    };
  }, [onRouteChange, events, open]);

  useEffect(() => {
    if (open) {
      document.body.style.overflow = "hidden";
    } else {
      document.body.style.overflow = "auto";
    }
  }, [open]);

  const Element = ({ item }) => {
    if (item.type === "subMenu") {
      return (
        <Popover className="relative">
          <>
            <Popover.Button className="relative flex items-center tracking-wider text-xl group uppercase shrink-0 group focus:outline-none focus-visible:text-[#fdb813]">
              {(item.icon as Media) && (
                <div className="border-r mr-6 pr-6">
                  <Image
                    src={createImageUrl((item.icon as Media).url)}
                    alt={(item.icon as Media).alt}
                    width={item.icon.width}
                    height={item.icon.height}
                    className="h-8 w-8 text-white"
                  />
                </div>
              )}
              <span className="group-hover:text-[#fdb813] transition-colors font-display">
                {item.label || (item.link?.reference.value as Page | Post).title}
              </span>
            </Popover.Button>
            <Transition
              as={Fragment}
              enter="transition ease-out duration-200"
              enterFrom="opacity-0 translate-y-1"
              enterTo="opacity-100 translate-y-0"
              leave="transition ease-in duration-150"
              leaveFrom="opacity-100 translate-y-0"
              leaveTo="opacity-0 translate-y-1"
            >
              <Popover.Panel className="absolute left-0 bottom-12 z-50 mt-3 w-full max-w-xs transform">
                <div className="overflow-hidden rounded-lg shadow-lg ring-1 ring-black ring-opacity-5">
                  <div className="relative grid gap-8 bg-white p-7">
                    {item.children?.map((item, index) => (
                      <Link
                        key={index}
                        href={
                          (websiteSettings.mainPage.value as Page).slug ===
                          (item.link.reference.value as Page | Post).slug
                            ? "/"
                            : getAbsolutePath((item.link?.reference.value as Page | Post).slug)
                        }
                        className="-m-3 flex items-center rounded-lg p-2 transition duration-150 ease-in-out hover:bg-gray-50 focus:outline-none focus-visible:ring focus-visible:ring-orange-500 focus-visible:ring-opacity-50"
                      >
                        {(item.link.icon as Media) && (
                          <div
                            className={joinClassNames(
                              "border-r flex rounded shrink-0 items-center justify-center sm:h-10 sm:w-10"
                            )}
                          >
                            <Image
                              src={createImageUrl((item.link.icon as Media).url)}
                              alt={(item.link.icon as Media).alt}
                              width={item.link.icon.width}
                              height={item.link.icon.height}
                              className="mr-4 h-8 w-8"
                            />
                          </div>
                        )}
                        <div className="ml-4">
                          <p className="text-gray-900 font-display text-lg tracking-wider">
                            {item.link.label || (item.link?.reference.value as Page | Post).title}
                          </p>
                        </div>
                      </Link>
                    ))}
                  </div>
                </div>
              </Popover.Panel>
            </Transition>
          </>
        </Popover>
      );
    }
    if (item.type === "link") {
      return (
        <Link
          href={
            (websiteSettings.mainPage.value as Page).slug ===
            (item.link.reference.value as Page).slug
              ? "/"
              : getAbsolutePath((item.link?.reference.value as Page | Post).slug)
          }
          className="relative flex items-center tracking-wider text-xl group uppercase group"
        >
          {(item.icon as Media) && (
            <div className="border-r border-gray-300 mr-6 pr-6">
              <Image
                src={createImageUrl((item.icon as Media).url)}
                alt={(item.icon as Media).alt}
                width={item.icon.width}
                height={item.icon.height}
                className="h-8 w-8 text-white"
              />
            </div>
          )}
          <span className="group-hover:text-[#fdb813] transition-colors font-display">
            {item.label || (item.link.reference.value as Page | Post).title}
          </span>
        </Link>
      );
    }
  };

  return (
    <Modal isOpen={open} setIsOpen={setOpen} className="max-w-3xl" align="top">
      <Modal.Header>
        <Modal.Title>Hlavní menu</Modal.Title>
      </Modal.Header>
      <Modal.Body>
        <div className="flex h-full w-full">
          <div className="mx-auto w-full max-w-5xl">
            <nav className="flex flex-col space-y-8">
              {items?.map((item, index) => (
                <Element item={item} key={index} />
              ))}
            </nav>
          </div>
        </div>
      </Modal.Body>
    </Modal>
  );
};

export default Menu;
