import { Dialog, Transition } from "@headlessui/react";
import { XMarkIcon } from "@heroicons/react/24/solid";
import { createContext, Dispatch, Fragment, ReactNode, SetStateAction } from "react";

import { joinClassNames } from "utils/helper";

type Props = {
  children: ReactNode[];
  isOpen: boolean;
  setIsOpen?: Dispatch<SetStateAction<boolean>>;
  className?: string;
  align: keyof typeof ALIGN_MAP;
};

const ALIGN_MAP = {
  top: "h-auto",
  middle: "h-full",
};

const ModalContext = createContext(null);

const Modal = ({ children, isOpen, setIsOpen, className, align }: Props) => {
  return (
    <Transition appear show={isOpen} as={Fragment}>
      <Dialog as="div" className="relative z-50" onClose={() => setIsOpen && setIsOpen(false)}>
        <Transition.Child
          as={Fragment}
          enter="ease-out duration-300"
          enterFrom="opacity-0"
          enterTo="opacity-100"
          leave="ease-in duration-200"
          leaveFrom="opacity-100"
          leaveTo="opacity-0"
        >
          <div className="fixed inset-0 bg-black bg-opacity-50 h-screen" />
        </Transition.Child>

        <div className="fixed inset-0 overflow-y-auto">
          <div
            className={joinClassNames(
              "flex items-center justify-center p-2 text-center",
              ALIGN_MAP[align]
            )}
          >
            <Transition.Child
              as={Fragment}
              enter="ease-out duration-300"
              enterFrom="opacity-0 scale-95"
              enterTo="opacity-100 scale-100"
              leave="ease-in duration-200"
              leaveFrom="opacity-100 scale-100"
              leaveTo="opacity-0 scale-95"
            >
              <Dialog.Panel
                className={joinClassNames(
                  "w-full transform overflow-hidden rounded-lg bg-white text-left align-middle shadow-xl transition-all",
                  className
                )}
              >
                <ModalContext.Provider value={{ isOpen, setIsOpen }}>
                  {children}
                  {/* {children.map((child: ReactElement<any>) =>
                    cloneElement(child, { setIsOpen, isOpen })
                  )} */}
                </ModalContext.Provider>
              </Dialog.Panel>
            </Transition.Child>
          </div>
        </div>
      </Dialog>
    </Transition>
  );
};

const Title = ({ children }) => (
  <Dialog.Title as="p" className="text-3xl font-display text-gray-600">
    {children}
  </Dialog.Title>
);

type HeaderProps = {
  children: ReactNode | ReactNode[];
  isOpen?: boolean;
  setIsOpen?: Dispatch<SetStateAction<boolean>>;
};

const Header = ({ children, isOpen, setIsOpen }: HeaderProps) => (
  <div className="flex justify-between items-center border-b border-gray-300 px-8 py-6">
    {children}
    <button type="button" onClick={() => setIsOpen && setIsOpen(!isOpen)}>
      <XMarkIcon className="h-8 w-8 text-gray-600" />
    </button>
  </div>
);

const Body = ({ children }) => <div className="p-8 text-gray-600">{children}</div>;

const Footer = ({ children }) => (
  <div className="border-t border-gray-300 p-8 text-gray-600">{children}</div>
);

Modal.Title = Title;
Modal.Header = Header;
Modal.Body = Body;
Modal.Footer = Footer;
export default Modal;
