import { Popover, Transition } from "@headlessui/react";
import Image from "next/image";
import Link from "next/link";
import { GlobalContext } from "pages/_app";
import { Header, Media, Page, Post } from "payload-types";
import { Fragment, useContext } from "react";

import { createImageUrl, getAbsolutePath } from "utils/helper";
import { joinClassNames } from "utils/helper";

type Props = {
  items: Header["items"];
};

type ElementProps = {
  item: Header["items"][number];
};

const Navigation = ({ items }: Props) => {
  const { websiteSettings } = useContext(GlobalContext);

  const Element = ({ item }: ElementProps) => {
    if (item.type === "subMenu") {
      return (
        <Popover className="relative shrink-0">
          <Popover.Button className="relative flex items-center text-white tracking-wider text-2xl group uppercase group focus:outline-none focus-visible:text-[#fdb813]">
            {(item.icon as Media) && (
              <Image
                width="0"
                height="0"
                sizes="100vw"
                src={createImageUrl((item.icon as Media).url)}
                alt={(item.icon as Media).alt}
                className="mr-4 h-8 w-8 text-white"
              />
            )}
            <span className="group-hover:text-[#fdb813] transition-colors font-display">
              {item.label || (item.link.reference.value as Page | Post).title}
            </span>
          </Popover.Button>
          {item.children && (
            <Transition
              as={Fragment}
              enter="transition ease-out duration-200"
              enterFrom="opacity-0 translate-y-1"
              enterTo="opacity-100 translate-y-0"
              leave="transition ease-in duration-150"
              leaveFrom="opacity-100 translate-y-0"
              leaveTo="opacity-0 translate-y-1"
            >
              <Popover.Panel className="absolute left-1/2 z-50 mt-3 w-screen max-w-xs -translate-x-1/2 transform px-4 sm:px-0">
                <div className="overflow-hidden rounded-lg shadow-lg ring-1 ring-black ring-opacity-5">
                  <div className="relative grid gap-8 bg-white p-7">
                    {item.children.map((item, index) => {
                      return (
                        item.link && (
                          <Link
                            key={index}
                            href={
                              (websiteSettings.mainPage.value as Page).slug ===
                              (item.link.reference.value as Page | Post).slug
                                ? "/"
                                : getAbsolutePath((item.link?.reference.value as Page | Post).slug)
                            }
                            className="-m-3 flex items-center rounded-lg p-2 transition duration-150 ease-in-out hover:bg-gray-50 focus:outline-none focus-visible:ring focus-visible:ring-orange-500 focus-visible:ring-opacity-50"
                          >
                            {(item.link.icon as Media) && (
                              <div
                                className={joinClassNames(
                                  "border-r flex rounded shrink-0 items-center justify-center text-white sm:h-10 sm:w-10"
                                )}
                              >
                                <Image
                                  src={createImageUrl((item.link.icon as Media).url)}
                                  alt={(item.link.icon as Media).alt}
                                  className="mr-4 h-8 w-8 text-white"
                                  width="0"
                                  height="0"
                                  sizes="100vw"
                                />
                              </div>
                            )}
                            <div className="ml-4">
                              <p className="text-gray-900 font-display text-lg tracking-wider">
                                {item.link.label ||
                                  (item.link?.reference.value as Page | Post).title}
                              </p>
                            </div>
                          </Link>
                        )
                      );
                    })}
                  </div>
                </div>
              </Popover.Panel>
            </Transition>
          )}
        </Popover>
      );
    }
    if (item.type === "link") {
      return (
        <Link
          href={
            (websiteSettings.mainPage.value as Page).slug ===
            (item.link.reference.value as Page).slug
              ? "/"
              : getAbsolutePath((item.link?.reference.value as Page | Post).slug)
          }
          className="relative flex items-center text-white tracking-wider text-2xl group uppercase shrink-0 group"
        >
          {(item.icon as Media) && (
            <Image
              width="0"
              height="0"
              sizes="100vw"
              src={createImageUrl((item.icon as Media).url)}
              alt={(item.icon as Media).alt}
              className="mr-4 h-8 w-8 text-white"
            />
          )}
          <span className="group-hover:text-[#fdb813] transition-colors font-display">
            {item.label || (item.link.reference.value as Page | Post).title}
          </span>
        </Link>
      );
    }
  };

  return (
    <nav className="hidden lg:flex space-x-12 w-full justify-between">
      {items?.map((item, index) => (
        <Element item={item} key={index} />
      ))}
    </nav>
  );
};

export default Navigation;
