import { format, parseISO } from "date-fns";
import { GetStaticProps } from "next";
import { NextSeo } from "next-seo";
import Image from "next/image";
import { Media, Post, User } from "payload-types";
import { fetchPost, fetchPostSlugs } from "queries/posts";
import webConfig from "web.config";

import { createImageUrl } from "utils/helper";

import { Heading } from "components/Heading";
import { LayoutMain } from "components/LayoutMain";
import { PreviewBar } from "components/PreviewBar";
import { RenderBlocks } from "components/RenderBlocks";

type PageProps = {
  post: Post;
  preview: boolean;
};

const Page = ({ post, preview }: PageProps) => {
  return (
    <>
      <NextSeo title={post.meta.title} description={post.meta.description} />
      {preview && <PreviewBar />}
      <div className="max-w-5xl mx-auto mt-8 px-2 xl:px-0">
        <Heading text="Článek" as="p" type="h1" marginBottom="xs"></Heading>
      </div>
      <div className="max-w-6xl mx-auto px-2 xl:px-0">
        <div className="relative">
          <div className="h-[16rem] sm:h-[20rem] md:h-[30rem] lg:h-[40rem]">
            <Image
              src={createImageUrl((post.image as Media).url)}
              alt={(post.image as Media).alt}
              className="rounded-xl fade w-full object-cover"
              sizes="(max-width: 768px) 100vw,
                (max-width: 1200px) 80vw,
                40vw"
              fill
            />
          </div>
          <div className="flex flex-col md:flex-row justify-between md:items-center relative -mt-16 max-w-5xl mx-auto px-2">
            <div>
              <Heading text={post.title} as="h1" type="h2"></Heading>
              <div className="flex flex-col">
                <div className="flex space-x-4 items-center">
                  <p
                    className="font-display text-lg md:text-2xl text-primary-600"
                    suppressHydrationWarning
                  >
                    Publikováno: {format(parseISO(post.createdAt), webConfig.dateFormat)}
                  </p>
                </div>
              </div>
            </div>
            <div className="flex items-center space-x-4">
              <p className="font-display md:text-2xl text-gray-600">
                Napsal {(post.author as User).name}
              </p>
              <div className="relative w-8 h-8 md:h-12 md:w-12">
                <Image
                  src={((post.author as User).image as Media).url}
                  className="rounded-full"
                  alt={((post.author as User).image as Media).alt}
                  sizes="10vw"
                  fill
                />
              </div>
            </div>
          </div>
        </div>
      </div>
      <RenderBlocks blocks={post.content} />
    </>
  );
};

export const getStaticProps: GetStaticProps<{ post: Post }> = async ({
  params,
  preview = false,
}) => {
  const post: Post = await fetchPost(params.slug as string, preview);
  if (!post) {
    return {
      notFound: true,
    };
  }

  return {
    props: { post, preview },
  };
};

export const getStaticPaths = async () => {
  const slugs = await fetchPostSlugs();
  return {
    paths: slugs?.map((slug: string) => `/blog/${slug}`),
    fallback: "blocking",
  };
};

Page.layout = LayoutMain;
export default Page;
